"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LightAbility = void 0;
const base_1 = require("./base");
class LightAbility extends base_1.Ability {
    /**
     * @param component - The light component to control.
     */
    constructor(component) {
        super(`Light ${component.id + 1}`, `light-${component.id}`);
        this.component = component;
    }
    get serviceClass() {
        return this.Service.Lightbulb;
    }
    initialize() {
        // set the initial value
        this.service.setCharacteristic(this.Characteristic.On, this.component.output);
        // listen for commands from HomeKit
        this.service.getCharacteristic(this.Characteristic.On)
            .onSet(this.onSetHandler.bind(this));
        this.service.getCharacteristic(this.Characteristic.Brightness)
            .onSet(this.brightnessSetHandler.bind(this));
        // listen for updates from the device
        this.component.on('change:output', this.outputChangeHandler, this);
        this.component.on('change:brightness', this.brightnessChangeHandler, this);
    }
    detach() {
        this.component.off('change:output', this.outputChangeHandler, this);
        this.component.off('change:brightness', this.brightnessChangeHandler, this);
    }
    /**
     * Handles changes to the Light.On characteristic.
     */
    async onSetHandler(value) {
        if (value === this.component.output) {
            return;
        }
        try {
            await this.component.set(value);
        }
        catch (e) {
            this.log.error('Failed to set light:', e instanceof Error ? e.message : e);
            throw -70402 /* this.api.hap.HAPStatus.SERVICE_COMMUNICATION_FAILURE */;
        }
    }
    /**
     * Handles changes to the `output` property.
     */
    outputChangeHandler(value) {
        if (value) {
            this.log.info('Light Status(' + this.component.id + '): on');
        }
        else {
            this.log.info('Light Status(' + this.component.id + '): off');
        }
        this.service.getCharacteristic(this.Characteristic.On)
            .updateValue(value);
    }
    /**
     * Handles changes to the Light.Brightness characteristic.
     */
    async brightnessSetHandler(value) {
        if (value === this.component.brightness) {
            return;
        }
        try {
            await this.component.set(undefined, value);
        }
        catch (e) {
            this.log.error('Failed to set light:', e instanceof Error ? e.message : e);
            throw -70402 /* this.api.hap.HAPStatus.SERVICE_COMMUNICATION_FAILURE */;
        }
    }
    /**
     * Handles changes to the `brightness` property.
     */
    brightnessChangeHandler(value) {
        this.log.info('Light Status(' + this.component.id + '): ' + value);
        this.service.getCharacteristic(this.Characteristic.Brightness)
            .updateValue(value);
    }
}
exports.LightAbility = LightAbility;
//# sourceMappingURL=light.js.map